#include "esp_system.h"
#include <WiFi.h>
#include <Wire.h>
#include <WiFiUdp.h>
#include <Adafruit_PWMServoDriver.h>
#include "SSD1306.h"//ディスプレイ用ライブラリを読み込み
                    
#define SSID "**********"          // 無線LANアクセスポイントのSSID
#define PASS "**********"              // パスワード
#define R_PORT 9000
#define MAXSN 12
#define SERVOMIN 100  // this is the 'minimum' pulse length count (out of 4096)
#define SERVOMAX 410  // this is the 'maximum' pulse length count (out of 4096)
#define FREQ 42   // PWM frequency
SSD1306 display(0x3c, 21, 22); // /SSD1306インスタンスの作成（I2Cアドレス,SDA,SCL）
WiFiServer server(80);
WiFiUDP udp;
// called this way, it uses the default address 0x40
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

int ledR=18;
int ledG=4;
int ledB=5;
int brt=64;
int ch1=12;
int ch2=13;
int ch3=14;
int nservo;
int dly0=500;
int dly1=500;
int dly2=1000;
int delta=30;
String lip;
String msg="";

//  Servo Position
int sPos[3][12]={
  {70,70,130,60,  60,40,110,150,  70,80,70,80},
  {90,90,40,90,  90,130,60,110,  90,90,90,90},
  {110,110,40,120, 120,130,60.180, 110,100,100,90}
};

// Fine angle adjustments (degrees)
int trim[MAXSN] = { 0,  // Head yaw         pin 0
                    0,  // Waist yaw        pin 1
                    0,  // R Sholder roll   pin 2
                    0,  // R Sholder pitch  pin 3
                                        
                    0,  // R Hand grip      pin 4
                    0,  // L Sholder roll   pin 5
                    0,  // L Sholder pitch  pin 6
                    0,  // L Hand grip      pin 7
                    
                    0,  // R Foot yaw       pin 8
                    0,  // R Foot pitch     pin 9
                    0,  // L Foot yaw       pim10
                    0}; // L Foot pitch     pin11
                    
void drawLcd(String s){
  display.init();    //ディスプレイを初期化
  display.setFont(ArialMT_Plain_16);    //フォントを設定
  display.drawString(0, 0, lip);
  display.drawString(0, 30,"Servo NO= "+s);
  display.display();   //指定された情報を描画
}

void setup() {
  Serial.begin(9600);
  WiFi.mode(WIFI_STA);                    // 無線LANをSTAモードに設定
  WiFi.begin(SSID,PASS);                  // 無線LANアクセスポイントへ接続
  while(WiFi.status() != WL_CONNECTED){   // 接続に成功するまで待つ
    Serial.println("Not Connected");
    delay(dly0);                           // 待ち時間処理
  }
  Serial.println("Connected successfully");
  
  lip=WiFi.localIP().toString();   // IPアドレスを取得
  drawLcd("Ready");
  udp.begin(R_PORT);
  pwm.begin();
  ledcAttachPin(ledR, ch1); // assign RGB led pins to channels
  ledcAttachPin(ledG, ch2);
  ledcAttachPin(ledB, ch3);
  
  ledcSetup(ch1, 12000, 8); // 12 kHz PWM, 8-bit resolution
  ledcSetup(ch2, 12000, 8);
  ledcSetup(ch3, 12000, 8);
  ledOn(0);
  pwm.setPWMFreq(FREQ); // Abnalog servos run at 60 Hz updates
  for (int i=0; i<MAXSN; i++){
    drawLcd("init"+String(i));
    servoWrite(i,sPos[1][i]);
    delay(dly1);
  }
  delay(dly2);

}

void loop() { // 
  char lcdp[20];                          // 表示用変数を定義(49バイト48文字)
  int len;                                // 文字列長を示す整数型変数を定義
  memset(lcdp, 0, 20);                    // 文字列変数lcdの初期化(49バイト)
  len = udp.parsePacket();                // 受信パケット長を変数lenに代入
  if(len>0){ 
    udp.read(lcdp, 20);
    // 受信データを文字列変数lcdpへ代入
    nservo=-1;
    char c=lcdp[0];
    if (c=='A') {nservo=0; msg="HEAD";}
    else if(c=='M') {nservo=1; msg="WAIST";}
    else if(c=='%') {nservo=2; msg="SH_Rr";}
    else if(c=='/') {nservo=3; msg="SH_Rp";}
    
    else if(c=='7') {nservo=4; msg="HND_R";} 
    else if(c=='8') {nservo=5; msg="SH_Lr";}
    else if(c=='9') {nservo=6; msg="SH_Lp";}
    else if(c=='*') {nservo=7; msg="HND_L";}

    else if(c=='4') {nservo=8; msg="FT_Ry";} 
    else if(c=='5') {nservo=9; msg="FT_Rp";}
    else if(c=='6') {nservo=10; msg="FT_Ly";}
    else if(c=='-') {nservo=11; msg="FT_Lp";}
    
    else if(c=='1') {nservo=12; msg="LED";} 
    msg=String(nservo);
    if (nservo>=0 && nservo<MAXSN){
      servoWrite(nservo,sPos[0][nservo]);
      delay(1000);
      servoWrite(nservo,sPos[2][nservo]);
      delay(1000);
      servoWrite(nservo,sPos[1][nservo]);
      drawLcd(msg);
    } else if(nservo==12){
      for (int j=1; j<8; j++) {
        ledOn(j);
        delay(dly2);
      }
      ledOn(0);
    }
  }
}

void ledOn(int c){
  int x=bitRead(c,0); ledcWrite(ch1,x*brt);
  x=bitRead(c,1); ledcWrite(ch2,x*brt);
  x=bitRead(c,2); ledcWrite(ch3,x*brt);
}

void servoWrite( int n, int a) {
// 回転させる角度に補正値を加算してから、PWM信号を変化させるときの値に変換
  int deg=a+trim[n];
  if (deg<0) deg=0;
  else if (deg>180) deg=180;
  int hrz = map( deg, 0, 180, SERVOMIN, SERVOMAX );
// サーボモータを回転
  pwm.setPWM( n, 0, hrz);
}
